########################################################
# Estimate seismic losses.
# Python tools for cmd. 
# 
# Usage:
# 1. loss assessment based on a dynamic analysis
#   --EQRecordFile <> --EQScaling <> --NumofStories <> --FloorArea <> --StructuralType <>
#       --OccupancyClass <> --DesignLevel <> --OutputDir <> --SelfCenteringEnhancingFactor <>
# 
# 2. Simulate EDP given a IM based on IDA results.
#   --IM_list <0.1 0.2 0.3 ...> --N_Sim <100> --IDA_result <> --betaM <>
#       --OutputDir <> --NumofStories <> --FloorArea <> --StructuralType <>
#       --OccupancyClass <> --DesignLevel <>
# 
# Dependancy: 
# - pandas, numpy, openseespy
########################################################

import argparse
import sys
from pathlib import Path

import pandas as pd
import numpy as np

from . import MDOF_LU as mlu
from . import MDOFOpenSees as mops
from . import BldLossAssessment as bl
from . import IDA
from . import Alpha_CNcode as ACN

def DynamicAnalysis_1Sim(NumofStories,FloorArea,StructuralType,OccupancyClass,
    DesignInfo,EQRecordFile,EQScaling,OutputDir,SelfCenteringEnhancingFactor):

    if DesignInfo['Code'] == 'Hazus':
        bld = mlu.MDOF_LU(NumofStories, FloorArea, StructuralType, 
                          SeismicDesignLevel=DesignInfo['SeismicDesignLevel'])
    elif DesignInfo['Code'] == 'CN':
        bld = mlu.MDOF_CN(NumofStories, FloorArea, StructuralType, 
            SeismicDesignLevel=DesignInfo['SeismicDesignLevel'], 
            EQGroup=DesignInfo['EQgroup'], 
            SiteClass=DesignInfo['SiteClass'])  
    else:
        print('ERROR: wrong DesignInfo')
        return
    # bld.OutputStructuralParameters('structural parameters')

    fe = mops.MDOFOpenSees(NumofStories, [bld.mass]*bld.N, [bld.K0]*bld.N, bld.DampingRatio,
        bld.HystereticCurveType, bld.Vyi, bld.betai, bld.etai, bld.DeltaCi, bld.tao)
    fe.SelfCenteringEnhancingFactor = SelfCenteringEnhancingFactor
    fe.DynamicAnalysis(EQRecordFile, EQScaling)
    # fe.PlotForceDriftHistory(1)

    DesignLevel = DesignInfo['SeismicDesignLevel']
    if DesignInfo['Code'] == 'CN':
        DesignLevel = ACN.Concert_CN2Hazus_SeismicDesignLevel(DesignInfo['SeismicDesignLevel'])
    blo = bl.BldLossAssessment(NumofStories, FloorArea,StructuralType,DesignLevel,OccupancyClass)
    blo.LossAssessment([fe.MaxDrift.max()],[fe.MaxAbsAccel.max()/9.8])  

    data = {
        'DS_Struct': blo.DS_Struct, 
        'DS_NonStruct_DriftSen':blo.DS_NonStruct_DriftSen,
        'DS_NonStruct_AccelSen': blo.DS_NonStruct_AccelSen,
        'RepairCost_Total': blo.RepairCost_Total,
        'RepairCost_Struct': blo.RepairCost_Struct,
        'RepairCost_NonStruct_DriftSen': blo.RepairCost_NonStruct_DriftSen,
        'RepairCost_NonStruct_AccelSen': blo.RepairCost_NonStruct_AccelSen,
        'RepairTime': blo.RepairTime,
        'RecoveryTime': blo.RecoveryTime,
        'FunctionLossTime': blo.FunctionLossTime
    }
    df = pd.DataFrame(data)
    df.to_csv(Path(OutputDir).joinpath('BldLoss.csv'),index=0)

def Simulate_losses_given_IM_basedon_IDA(IDA_result,IM_list,N_Sim,betaM,OutputDir,
    NumofStories,FloorArea,StructuralType,DesignInfo,OccupancyClass) -> tuple[pd.DataFrame,pd.DataFrame]:

    IDA_result = pd.read_csv(Path(IDA_result))
    IDA_result['Iffinish'] = IDA_result['Iffinish'].astype(bool)
    IDA_result = IDA_result.loc[:, ~IDA_result.columns.str.contains('^Unnamed')]
    IDA_result = IDA_result.loc[IDA_result['Iffinish']==1,:]  
    for ind,row in IDA_result.iterrows():
        for varname in ['MaxDrift','MaxAbsAccel','MaxRelativeAccel']:
            if isinstance(row[varname],str):
                IDA_result.at[ind,varname] = np.array(
                    [float(val) for val in row[varname].replace('[','').replace(']','').split()])

    if len(N_Sim)==1:
        N_Sim = N_Sim[0]
    SimEDP = IDA.SimulateEDPGivenIM(IDA_result,IM_list,N_Sim,betaM)
    if OutputDir is not None:
        SimEDP.to_csv(Path(OutputDir)/'SimEDP.csv')

    DesignLevel = DesignInfo['SeismicDesignLevel']
    if DesignInfo['Code'] == 'CN':
        DesignLevel = ACN.Concert_CN2Hazus_SeismicDesignLevel(DesignInfo['SeismicDesignLevel'])
    blo = bl.BldLossAssessment(NumofStories, FloorArea,StructuralType,DesignLevel,OccupancyClass)
    blo.LossAssessment(SimEDP['MaxDrift'].tolist(),(SimEDP['MaxAbsAccel']/9.8).tolist(),
        SimEDP['ResDrift'].tolist())  
    data = {
        'IM': SimEDP['IM'].tolist(),
        'DS_Struct': blo.DS_Struct, 
        'DS_NonStruct_DriftSen':blo.DS_NonStruct_DriftSen,
        'DS_NonStruct_AccelSen': blo.DS_NonStruct_AccelSen,
        'RepairCost_Total': blo.RepairCost_Total,
        'RepairCost_Struct': blo.RepairCost_Struct,
        'RepairCost_NonStruct_DriftSen': blo.RepairCost_NonStruct_DriftSen,
        'RepairCost_NonStruct_AccelSen': blo.RepairCost_NonStruct_AccelSen,
        'RepairTime': blo.RepairTime,
        'RecoveryTime': blo.RecoveryTime,
        'FunctionLossTime': blo.FunctionLossTime
    }
    df = pd.DataFrame(data)
    if OutputDir is not None:
        df.to_csv(Path(OutputDir)/'BldLoss.csv')

    return SimEDP, df


def main(args):
    parser = argparse.ArgumentParser()
    
    # usage 1
    parser.add_argument('--EQRecordFile')
    parser.add_argument('--EQScaling',default = 1.0,type=float)
    parser.add_argument('--SelfCenteringEnhancingFactor',
        default = 0, type=float)

    # usage 2   
    parser.add_argument('--IM_list', nargs='+', type=float)
    parser.add_argument('--N_Sim', nargs='+', type=int)
    parser.add_argument('--IDA_result')
    parser.add_argument('--betaM', type=float, default=0.0)

    # common arguments
    parser.add_argument('--OutputDir',default = '')
    parser.add_argument('--NumofStories',type=int)
    parser.add_argument('--FloorArea',type=float)
    parser.add_argument('--StructuralType')
    parser.add_argument('--OccupancyClass')
    parser.add_argument('--DesignInfo', type=dict)

    args = parser.parse_args(args)

    if not args.EQRecordFile is None:
        DynamicAnalysis_1Sim(args.NumofStories,args.FloorArea,args.StructuralType,
            args.OccupancyClass,args.DesignInfo,args.EQRecordFile,
            args.EQScaling,args.OutputDir,args.SelfCenteringEnhancingFactor)
    elif not args.IDA_result is None:
        Simulate_losses_given_IM_basedon_IDA(args.IDA_result,args.IM_list,args.N_Sim,
            args.betaM,args.OutputDir,
            args.NumofStories,args.FloorArea,args.StructuralType,args.DesignInfo,args.OccupancyClass)
    else:
        print('ERROR: wrong arguments')

# test function
# Simulate_losses_given_IM_basedon_IDA('E:/CityResilienceAndResilientStructure/IDA_results/IDA_result_ReprBldID_2750.csv',
#     [0.597312331199646, 0.7504111528396606, 0.11677270382642746, 1.0826855897903442, 0.7678610682487488, 0.31554660201072693, 0.21094414591789246, 0.41849640011787415, 0.4890880286693573, 0.3715813457965851, 0.9597485065460205, 0.6469749808311462, 0.312572717666626, 0.23771028220653534, 1.075209140777588, 0.572037398815155, 0.22335407137870789, 0.7163649797439575, 0.9657233953475952, 1.1582597494125366, 0.3978790044784546, 0.6954623460769653, 0.16181659698486328, 0.45524513721466064, 0.5643849968910217, 0.6057125926017761, 0.5204574465751648, 1.6083588600158691, 0.6610217690467834, 0.4120872914791107, 0.3005894124507904, 0.5215364694595337, 0.3321076035499573, 0.17521749436855316, 0.34796813130378723, 1.1377407312393188, 0.515545129776001, 0.5572354793548584, 0.5253138542175293, 0.9321489334106445, 0.24939264357089996, 0.2676057517528534, 0.6033239364624023, 0.3066806495189667, 0.26848989725112915, 0.5706826448440552, 0.3907144367694855, 0.828408420085907, 1.2435253858566284, 0.5493733286857605, 0.6190784573554993, 1.7332799434661865, 0.23518775403499603, 0.721286952495575, 0.2761167585849762, 0.4273426830768585, 0.34175413846969604, 0.3736478388309479, 0.3383278548717499, 0.6811069846153259, 0.6044138669967651, 0.1518346518278122, 0.3264392912387848, 0.7460941076278687, 0.8386232256889343, 0.43991437554359436, 0.4704597294330597, 0.368545800447464, 0.9768068194389343, 0.12327594310045242, 0.3202933669090271, 0.5888903737068176, 0.37826433777809143, 0.7602016925811768, 0.4129345118999481, 0.4118672311306, 0.8132396936416626, 0.7810140252113342, 0.575344979763031, 0.6012459993362427, 0.8843552470207214, 0.7849920392036438, 0.25877800583839417, 0.4878627359867096, 0.4558645784854889, 0.5710114240646362, 0.3063179850578308, 2.122384548187256, 0.5848923921585083, 0.35794347524642944, 0.12979377806186676, 0.8750959634780884, 0.18860745429992676, 0.3200041353702545, 0.3555286228656769, 0.19630251824855804, 0.920133113861084, 0.3903917968273163, 0.6425352692604065, 0.9717804193496704, 0.40658247470855713, 0.10220140963792801, 0.2998770475387573, 0.5437259078025818, 0.3062310814857483, 1.4468364715576172, 0.3366183340549469, 0.34343796968460083, 0.09149225056171417, 0.7806566953659058, 0.7522894740104675, 1.1283740997314453, 0.43889427185058594, 1.006947636604309, 0.29405152797698975, 0.4145025610923767, 0.3544003963470459, 0.34026554226875305, 0.35136136412620544, 0.514676034450531, 0.48821789026260376, 2.322253465652466, 0.34902292490005493, 1.239992380142212, 0.4800032079219818, 1.367623209953308, 0.9311930537223816, 0.3209489583969116, 0.17923001945018768, 0.7254462242126465, 0.33391404151916504, 0.5382861495018005, 0.926424503326416, 0.20374128222465515, 0.24788130819797516, 0.35000836849212646, 0.3844868242740631, 0.4902685284614563, 1.1849559545516968, 1.054950475692749, 0.3944760859012604, 1.5150240659713745, 0.5826113224029541, 0.3130607604980469, 1.116371989250183, 0.5177631378173828, 0.2096397429704666, 0.8316940665245056, 0.4205661714076996, 0.3375413715839386, 0.5111217498779297, 0.4825564920902252, 0.1382029950618744, 0.3659776747226715, 0.24046723544597626, 0.7485821843147278, 0.822441816329956, 0.73796546459198, 1.932750940322876, 0.7331615090370178, 0.6732274889945984, 1.8568689823150635, 0.9343371987342834, 0.6188607215881348, 0.6834572553634644, 0.37109512090682983, 0.48077699542045593, 0.617924153804779, 0.3982751667499542, 0.765853762626648, 0.28152546286582947, 1.115195393562317, 0.4284157454967499, 0.2452252060174942, 0.8876279592514038, 0.36466872692108154, 0.9826443791389465, 0.6285382509231567, 0.8957759737968445, 0.9921523332595825, 0.6455821990966797, 0.5177205204963684, 0.7712262868881226, 0.2810862362384796, 0.5895934104919434, 0.7188895344734192, 0.4397549331188202, 0.4907121956348419, 0.5646783113479614, 0.11033209413290024, 0.36438900232315063, 0.9387779235839844, 1.2611730098724365, 0.3413141667842865, 0.3168528079986572, 0.34288647770881653, 0.4215441346168518, 0.5796149969100952, 0.3733291029930115, 0.557776689529419, 1.8790910243988037, 0.4713607728481293, 0.4501644968986511, 0.41646358370780945, 0.9104756116867065, 0.47842177748680115, 0.5994683504104614, 0.23906686902046204, 0.9706763029098511, 1.11089289188385, 0.5746179223060608, 0.6875240802764893, 0.209690660238266, 0.946364164352417, 0.8896855115890503, 0.7960214614868164, 0.12730398774147034, 0.3658604323863983, 0.47817909717559814, 0.4159741997718811, 0.14219194650650024, 0.4048636257648468, 0.6662976145744324, 0.45738640427589417, 0.6049138903617859, 0.23375387489795685, 0.7818849086761475, 0.22861427068710327, 1.115351676940918, 0.35364240407943726, 0.24183666706085205, 0.9803659319877625, 0.5603623986244202, 0.40391242504119873, 0.8063054084777832, 0.5214652419090271, 0.2719319760799408, 0.7735315561294556, 0.4159816801548004, 0.3091653287410736, 0.3850117623806, 1.2443276643753052, 0.5802837014198303, 0.2509858012199402, 0.23596306145191193, 0.856197714805603, 0.38471537828445435, 0.5823880434036255, 1.5133854150772095, 0.9898894429206848, 0.12481079995632172, 0.736936628818512, 0.10098090022802353, 0.48322606086730957, 0.9813981056213379, 0.588591456413269, 0.3658030331134796, 0.4362272620201111, 0.8648843169212341, 0.9019734263420105, 0.8467634916305542, 0.8171064853668213, 0.5773541927337646, 1.3586952686309814, 0.757895827293396, 0.48250699043273926, 0.15603230893611908, 0.2203751653432846, 1.6359421014785767, 0.283054381608963, 0.3955775499343872, 1.171127438545227, 0.17752397060394287, 0.6793251037597656, 0.640631914138794, 0.32827645540237427, 0.5091416239738464, 1.2949837446212769, 0.5748385190963745, 0.48887190222740173, 0.29090720415115356, 0.3551558256149292, 0.488772451877594, 1.0043116807937622, 0.17637142539024353, 1.6523048877716064, 0.6974209547042847, 1.158385157585144, 0.596592903137207, 0.4142701029777527, 1.1332396268844604, 0.19159041345119476, 0.3654097616672516, 1.3987771272659302, 0.6081658005714417, 0.20512045919895172, 0.41314300894737244, 1.7113341093063354, 0.7632458806037903, 0.3266485035419464, 0.7656493782997131, 0.43912291526794434, 1.6728854179382324, 0.36105456948280334, 0.18673843145370483, 0.6418051719665527, 0.7553209662437439, 0.4930533170700073, 0.5481774806976318, 0.6612897515296936, 0.24192871153354645, 0.9692611694335938, 0.3181951940059662, 0.5781452059745789, 2.163013458251953, 0.7091037034988403, 0.4299466907978058, 0.3695445656776428, 0.5343793034553528, 0.26587975025177, 0.2648836672306061, 0.3703822195529938, 0.33784225583076477, 1.050249457359314, 0.09356192499399185, 0.4920822083950043, 0.4304668605327606, 0.3902725577354431, 0.44056040048599243, 0.8994930982589722, 0.26901715993881226, 1.3927346467971802, 0.45531362295150757, 0.6605846285820007, 0.4974544942378998, 0.43259766697883606, 0.4195937216281891, 0.40116235613822937, 0.2765696346759796, 0.5278375148773193, 0.4588747024536133, 0.2804584205150604, 1.4830646514892578, 0.6359775066375732, 0.40881577134132385, 0.20303718745708466, 0.8868913650512695, 0.20783300697803497, 0.7779678702354431, 0.1854901909828186, 0.5173425078392029, 0.5045576691627502, 1.0973707437515259, 0.18924790620803833, 0.27761194109916687, 0.48575645685195923, 0.08517047762870789, 0.3966335356235504, 0.3574061989784241, 1.0526390075683594, 0.22586244344711304, 1.3428462743759155, 0.7167506814002991, 0.5716546773910522, 0.528399646282196, 1.0787009000778198, 1.0116896629333496, 0.319004088640213, 0.6391639113426208, 0.26502013206481934, 0.2312346249818802, 1.4699270725250244, 0.42855390906333923, 0.3949331045150757, 0.6436690092086792, 0.23682524263858795, 1.1071672439575195, 0.31224822998046875, 0.374083936214447, 0.4164070188999176, 0.29695409536361694, 0.33765825629234314, 1.019535779953003, 0.46339988708496094, 0.3670065999031067, 0.3189626932144165, 0.6386035084724426, 0.377162367105484, 0.7191930413246155, 1.0451369285583496, 0.8954559564590454, 0.6224087476730347, 0.970851480960846, 0.4990018308162689, 0.18255099654197693, 0.5610021948814392, 1.7766414880752563, 0.5034230947494507, 0.33244645595550537, 1.2374736070632935, 0.4107356369495392, 0.39659997820854187, 0.6805306673049927, 1.0483362674713135, 0.4483312666416168, 0.2118762582540512, 1.3041176795959473, 0.6813960671424866, 0.23657143115997314, 0.4644668698310852, 1.0176682472229004, 0.2253991812467575, 0.4670884311199188, 0.6348043084144592, 1.0384361743927002, 0.8004604578018188, 0.6697429418563843, 2.9282681941986084, 0.34635043144226074, 0.9462570548057556, 0.3934013545513153, 1.035873532295227, 0.7965509295463562, 1.2112445831298828, 0.48554128408432007, 0.3829313814640045, 0.7862005829811096, 0.5494223237037659, 0.26474568247795105, 0.5173794031143188, 0.18896597623825073, 1.6345702409744263, 0.24344393610954285, 0.20666322112083435, 0.31318408250808716, 0.3160235583782196, 0.14905108511447906, 0.9264643788337708, 0.5249994397163391, 0.6890904307365417, 1.5893266201019287, 1.231216549873352, 0.21404002606868744, 0.2605758011341095, 1.2787580490112305, 0.23551863431930542, 0.19230389595031738, 0.37865573167800903, 0.8799127340316772, 1.929793357849121, 0.11288481950759888, 0.7518848180770874, 0.2762850522994995, 0.2622424066066742, 0.44825276732444763, 0.14508426189422607, 0.8192092180252075, 0.18348456919193268, 0.2871652841567993, 0.7239206433296204, 0.3257825970649719, 0.8675707578659058, 0.24061213433742523, 0.09034783393144608, 0.5386753082275391, 0.17258676886558533, 0.5479644536972046, 0.488307923078537, 0.52581787109375, 0.44688156247138977, 0.6149769425392151, 0.14782093465328217, 0.5529321432113647, 0.4587123692035675, 0.44080913066864014, 0.5611972212791443, 1.2813630104064941, 0.2506471574306488, 1.0435152053833008, 0.3465881049633026, 0.7267953753471375, 1.3067355155944824, 0.897661030292511, 0.3408459722995758, 0.3449915051460266, 0.3632858991622925, 2.2880184650421143, 0.6383892297744751, 0.16213113069534302, 0.5655331611633301, 0.6419585347175598, 0.5287418365478516, 0.2644180357456207, 0.38955220580101013, 0.4668503403663635, 0.23424135148525238, 
#     0.44844555854797363, 0.7909022569656372, 0.35616329312324524, 0.39032506942749023, 0.530531644821167, 0.8185354471206665, 0.2228594720363617, 1.8555046319961548, 0.4957239031791687, 0.6892781853675842, 1.1386034488677979, 0.2052985429763794, 0.7062844038009644, 1.5338127613067627, 0.5599177479743958, 0.40549540519714355, 0.7993192672729492, 1.8344999551773071, 0.6149519681930542, 0.36417967081069946, 0.6228287220001221, 0.4740417003631592, 0.6734341979026794, 0.3316677212715149, 0.9381093978881836, 0.3269018828868866, 0.4193301200866699, 0.47669127583503723, 0.2690940201282501, 0.4767202138900757, 0.9031187295913696, 1.1096487045288086, 0.27305683493614197, 0.5834334492683411, 0.5322328209877014, 0.6128847002983093, 0.6768479943275452, 0.6807469129562378, 0.5225727558135986, 0.8095136880874634, 0.8967778086662292, 0.8804548978805542, 0.22515836358070374, 0.5921615362167358, 0.7910623550415039, 0.5199098587036133, 0.13299088180065155, 0.2109345644712448, 1.268519401550293, 0.18573926389217377, 0.8827742338180542, 1.2075132131576538, 0.5435121655464172, 1.0490930080413818, 0.8857313394546509, 0.3321608901023865, 0.299408495426178, 0.36161282658576965, 1.2273417711257935, 0.5956721901893616, 0.6860693097114563, 0.1205904483795166, 0.34667229652404785, 0.2220650613307953, 1.086023211479187, 0.3936614394187927, 0.22479450702667236, 0.5901395678520203, 0.13436035811901093, 0.26213401556015015, 0.6958121061325073, 0.8364533185958862, 0.14393821358680725, 0.14241576194763184, 0.24679435789585114, 0.2885947525501251, 0.2391778826713562, 1.3323215246200562, 1.2881039381027222, 0.4952147603034973, 0.3843300938606262, 0.7408339381217957, 0.22886136174201965, 0.7389900088310242, 1.2703527212142944, 0.44135811924934387, 0.6636453866958618, 0.5614086985588074, 0.44296109676361084, 0.19642610847949982, 0.7335145473480225, 0.2517745792865753, 0.28327763080596924, 0.2656581401824951, 1.7018910646438599, 1.2790130376815796, 0.6380119919776917, 0.22053642570972443, 0.5382788181304932, 0.4155294597148895, 0.5179073214530945, 0.2750375270843506, 0.29395583271980286, 0.14722616970539093, 0.42859184741973877, 0.7601200938224792, 0.5692314505577087, 0.38896849751472473, 0.6832495927810669, 2.0450592041015625, 0.2560926377773285, 0.4072713553905487, 0.6235964894294739, 1.1995640993118286, 0.45618534088134766, 1.2983791828155518, 1.2698272466659546, 0.46103858947753906, 0.1871701180934906, 0.40721577405929565, 0.2769593596458435, 0.349415123462677, 0.8079590797424316, 0.8992587924003601, 0.6154683232307434, 0.618010938167572, 0.2588317394256592, 1.0170941352844238, 0.5758506059646606, 0.6327954530715942, 0.7208805680274963, 0.4405687153339386, 0.4112475514411926, 1.2977277040481567, 0.41432276368141174, 0.7847006916999817, 0.720138430595398, 1.5711158514022827, 0.3551361858844757, 1.7096811532974243, 0.7648978233337402, 0.7540821433067322, 2.1937084197998047, 0.4732884466648102, 1.7775602340698242, 0.44364461302757263, 0.16634170711040497, 0.2789253890514374, 0.37243255972862244, 0.6957323551177979, 0.2975938320159912, 0.4138249456882477, 0.46275386214256287, 1.0006614923477173, 0.2644140422344208, 0.29487907886505127, 1.1885145902633667, 0.3303394615650177, 0.7866466641426086, 0.6010622978210449, 0.6157982349395752, 1.0117474794387817, 1.130924105644226, 0.771518886089325, 0.537037193775177, 0.8591649532318115, 0.16088475286960602, 0.842512309551239, 0.6715758442878723, 0.6047567129135132, 0.818839430809021, 0.4248483180999756, 0.6937243342399597, 0.46029868721961975, 0.30045750737190247, 0.18137171864509583, 0.22110025584697723, 0.35079649090766907, 0.46781814098358154, 0.3033135235309601, 0.23332402110099792, 0.47646164894104004, 0.9515426754951477, 1.837823748588562, 0.37481623888015747, 0.8462706208229065, 0.31462252140045166, 0.34826305508613586, 0.7064304947853088, 0.3060547709465027, 1.3531935214996338, 0.35767754912376404, 0.5096468329429626, 0.44715824723243713, 0.12136758118867874, 0.501934826374054, 0.345201313495636, 0.4291656017303467, 1.7626906633377075, 0.14328858256340027, 0.797049880027771, 0.44391271471977234, 0.49080216884613037, 0.5297651886940002, 0.4008249044418335, 0.25094449520111084, 0.7982181310653687, 0.49256375432014465, 0.28166136145591736, 0.5152847170829773, 0.5144289135932922, 0.39614132046699524, 0.82576984167099, 0.5710644125938416, 0.3068414628505707, 0.9946376085281372, 0.3121299743652344, 0.24572190642356873, 0.6150885820388794, 1.1342698335647583, 1.3188689947128296, 0.42200374603271484, 1.2665989398956299, 0.5804616212844849, 1.3650851249694824, 0.3895663619041443, 0.3431640863418579, 0.38069555163383484, 0.3261745274066925, 0.29645541310310364, 0.48673877120018005, 1.3366104364395142, 0.8344743847846985, 0.19539575278759003, 1.5566515922546387, 0.44021299481391907, 0.6091410517692566, 0.4815160036087036, 0.26192793250083923, 0.48554727435112, 0.7258033156394958, 0.3228405714035034, 1.3542871475219727, 0.8082868456840515, 0.2447493076324463, 1.0763767957687378, 0.204896941781044, 0.8821212649345398, 0.44853028655052185, 0.5852249264717102, 0.4589575231075287, 0.3010319471359253, 0.1384483128786087, 0.14935295283794403, 0.5378891825675964, 0.14155356585979462, 0.4224076569080353, 0.7760393619537354, 0.2601846754550934, 0.8510890603065491, 0.4419686794281006, 1.5354859828948975, 0.6515029072761536, 1.2719879150390625, 0.3655368387699127, 0.36893531680107117, 0.32521310448646545, 0.7162860035896301, 0.5282987952232361, 0.23413968086242676, 0.9517584443092346, 0.44466423988342285, 0.31679895520210266, 0.27299296855926514, 0.4123765826225281, 0.3859504759311676, 0.8667944073677063, 0.7494161128997803, 0.41109350323677063, 1.1616262197494507, 0.6517348289489746, 0.2606223225593567, 0.4435729682445526, 1.550144076347351, 0.7654356956481934, 0.5732675790786743, 0.40265417098999023, 0.27456656098365784, 0.251693457365036, 0.797483503818512, 0.5594321489334106, 1.1789039373397827, 0.3733201324939728, 0.4739871025085449, 0.2640147805213928, 0.7542626261711121, 0.24998341500759125, 0.3527545928955078, 0.21635423600673676, 0.2719728350639343, 0.8631471395492554, 0.2546548545360565, 0.1911899447441101, 0.6065509915351868, 0.7472438216209412, 0.3231254816055298, 0.7290787696838379, 0.412935346364975, 0.25855129957199097, 0.1137041449546814, 0.9915522336959839, 0.724143922328949, 0.6531609892845154, 0.29754987359046936, 0.2766478955745697, 0.35777080059051514, 0.7802221775054932, 0.5321504473686218, 0.7528468370437622, 0.23383712768554688, 1.6777342557907104, 0.9387437701225281, 0.8198129534721375, 0.9177992939949036, 2.3031387329101562, 0.9538387656211853, 0.9315149784088135, 1.6916279792785645, 0.44059160351753235, 0.43009328842163086, 1.0348989963531494, 0.9610660672187805, 0.3690606355667114, 0.11979829519987106, 0.22747448086738586, 0.5449356436729431, 0.41339781880378723, 0.4841936528682709, 0.8077366352081299, 0.45225200057029724, 0.45960143208503723, 0.24901676177978516, 0.22430089116096497, 1.107283115386963, 0.5415158867835999, 0.6486895084381104, 0.9857988953590393, 0.5619622468948364, 0.5118358135223389, 0.19471848011016846, 1.0193084478378296, 0.28907641768455505, 1.0030626058578491, 0.5987786054611206, 0.6217333078384399, 0.2106577754020691, 0.40228602290153503, 0.24639169871807098, 0.5478689670562744, 0.7502709627151489, 0.3805556297302246, 1.0863542556762695, 1.4373358488082886, 0.4831753969192505, 0.3817645013332367, 0.8811827898025513, 0.37484145164489746, 0.2937634587287903, 0.4129604399204254, 0.5133458375930786, 0.833571195602417, 1.1798840761184692, 0.2330390214920044, 1.2242594957351685, 0.6498743891716003, 0.4913267493247986, 1.3311868906021118, 0.4745939075946808, 0.42248955368995667, 0.547234833240509, 0.8422767519950867, 0.48306378722190857, 1.200904369354248, 0.8209832310676575, 0.24405977129936218, 0.47445276379585266, 0.47101107239723206, 0.20551565289497375, 0.5401106476783752, 1.6203595399856567, 0.8855424523353577, 0.2482299506664276, 0.41442447900772095, 0.3021567165851593, 1.2743366956710815, 0.5691235065460205, 0.4869641661643982, 0.24568121135234833, 0.4164453148841858, 0.23035189509391785, 0.6400081515312195, 0.4783026874065399, 0.5648216605186462, 1.3392164707183838, 0.49983876943588257, 0.13296009600162506, 0.5166847109794617, 0.8886466026306152, 1.4569504261016846, 0.40626564621925354, 0.3044070601463318, 0.4715515077114105, 0.3395872712135315, 0.32903313636779785, 0.34141090512275696, 0.2269430160522461, 0.2651066184043884, 0.42974600195884705, 0.32305899262428284, 0.2975565195083618, 0.8680331110954285, 0.8546080589294434, 0.4120635986328125, 0.46063032746315, 0.23896683752536774, 0.6357845067977905, 1.8084748983383179, 0.5621764063835144, 0.7699961066246033, 1.6616530418395996, 0.21979083120822906, 0.3508036732673645, 0.8347534537315369, 0.5841401815414429, 1.0419495105743408, 0.6270683407783508, 0.7765610218048096, 0.4218120276927948, 0.862577497959137, 0.48327967524528503, 0.1674596667289734, 0.4884801208972931, 0.5922306776046753, 1.052188515663147, 0.5073641538619995, 0.39083436131477356, 0.28337499499320984, 1.0765020847320557, 0.2116016149520874, 0.3125368356704712, 0.6914190649986267, 1.0280689001083374, 0.4550918936729431, 0.7016298770904541, 0.4228419363498688, 0.5960544943809509, 0.1378246247768402, 0.26476600766181946, 0.2902599573135376, 0.43645235896110535, 0.8661947250366211, 1.5998501777648926, 1.544021725654602, 0.1875111609697342, 0.1408572643995285, 0.6063895225524902, 0.5890004634857178, 0.5907024145126343, 0.45318078994750977, 0.44992488622665405, 0.4313347637653351, 0.2144576460123062, 0.8166390061378479, 0.2728210985660553, 0.1846248209476471, 0.28885528445243835, 0.22294588387012482, 0.4252764582633972, 0.4889293611049652, 0.877134382724762, 0.49151483178138733, 0.41635662317276, 0.7694553732872009, 0.5088596343994141, 0.241668239235878, 0.3461660146713257, 0.33705946803092957, 0.49229270219802856, 0.22030173242092133, 0.44375938177108765, 0.8427075743675232, 0.6466205716133118, 0.8498249650001526, 0.549732506275177, 
#     0.4351741373538971, 0.3869691789150238, 1.4960206747055054, 0.8383181095123291, 0.8121029138565063, 0.4480784237384796, 1.0219601392745972, 1.3446862697601318, 0.6105179190635681, 0.5198213458061218],
#     [1],
#     0.25,
#     'E:/CityResilienceAndResilientStructure/RegionalLossSimulations',
#     4,
#     3593.5,
#     'C1',
#     {'Code': 'Hazus', 'SeismicDesignLevel': 'UNKNOWN'},
#     'RES1')


if __name__ == "__main__":
    main(sys.argv[1:])
